#
# A Named Entity Linker based on the Babel cliques using SAPBERT embeddings.
# Source code: https://github.com/RENCI-NER/sapbert
# Hosted at: https://sap-qdrant.apps.renci.org/docs
#
import requests

from renci_ner.core import (
    AnnotatedText,
    AnnotationProvenance,
    Annotator,
    NormalizedAnnotation,
)

# Configuration.
RENCI_SAPBERT_URL = "https://sap-qdrant.apps.renci.org"
DEFAULT_LIMIT = 10


class SAPBERTAnnotator(Annotator):
    """
    Provides an Annotator interface to a SAPBERT service.
    """

    @property
    def provenance(self) -> AnnotationProvenance:
        """Return an AnnotationProvenance describing annotations produced by this service."""
        return AnnotationProvenance(
            name="BabelSAPBERT", url=RENCI_SAPBERT_URL, version=self.openapi_version
        )

    def __init__(self, url=RENCI_SAPBERT_URL, requests_session=requests.Session()):
        """
        Set up a SAPBERT service.

        :param url: The URL of a SAPBERT service.
        :param requests_session: A Requests session object to use instead of the default one.
        """
        self.url = url
        self.annotate_url = url + "/annotate/"
        self.requests_session = requests_session

        response = requests.get(self.url + "/openapi.json")
        response.raise_for_status()
        openapi_data = response.json()
        self.openapi_version = openapi_data.get("info", {"version": "NA"}).get(
            "version", "NA"
        )

    def supported_properties(self):
        return {
            "limit": "The maximum number of results to return.",
            "score": "The (minimum) score for this result returned by SAPBERT (higher is better).",
        }

    def annotate(self, text, props={}) -> AnnotatedText:
        # Set up query.
        session = self.requests_session

        min_score = props.get("score", 0)
        limit = props.get("limit", DEFAULT_LIMIT)

        response = session.post(
            self.annotate_url,
            json={
                "text": text,
                "model_name": "sapbert",
                "count": limit,
            },
        )

        response.raise_for_status()
        results = response.json()

        # Find the first result that meets our criteria.
        annotations = []
        for result in results:
            if result.get("score", 0) < min_score:
                continue

            annotations.append(
                # Since SAPBERT is normalized to Babel, we can treat it as a NormalizedAnnotation.
                NormalizedAnnotation(
                    text=text,
                    curie=result.get("curie", ""),
                    label=result.get("name", ""),
                    biolink_type=result.get("category", ""),
                    id=result.get("curie", ""),
                    type=result.get("category", ""),
                    props={
                        "score": result.get("score", 0),
                    },
                    provenance=self.provenance,
                    # Since we're using the whole text, let's just use that
                    # as the start/end.
                    start=0,
                    end=len(text),
                )
            )

        return AnnotatedText(text, annotations)



#
# The Translator Node Normalizer.
# Source code: https://github.com/TranslatorSRI/NodeNormalization
# Hosted at: https://nodenormalization-sri.renci.org/
#
import logging

import requests

from renci_ner.core import (
    AnnotatedText,
    AnnotationProvenance,
    NormalizedAnnotation,
    Transformer,
)

# Configuration.
RENCI_NODENORM_URL = "https://nodenormalization-sri.renci.org"


class NodeNorm(Transformer):
    """
    The Translator Node Normalizer.
    """

    @property
    def provenance(self) -> AnnotationProvenance:
        """Return an AnnotationProvenance describing annotations produced by this service."""
        return AnnotationProvenance(
            name="NodeNorm", url=RENCI_NODENORM_URL, version=self.openapi_version
        )

    def __init__(self, url=RENCI_NODENORM_URL, requests_session=requests.Session()):
        """
        Set up a BioMegatron service.

        :param url: The URL of the BioMegatron service.
        :param requests_session: A Requests session object to use instead of the default one.
        """
        self.url = url
        self.get_normalized_nodes_url = url + "/get_normalized_nodes"
        self.requests_session = requests_session

        response = requests.get(self.url + "/openapi.json")
        response.raise_for_status()
        openapi_data = response.json()
        self.openapi_version = openapi_data.get("info", {"version": "NA"}).get(
            "version", "NA"
        )

    def supported_properties(self):
        """Some configurable parameters."""
        return {
            "geneprotein_conflation": "(true/false, default: true) Whether to conflate gene and protein identifiers.",
            "drugchemical_conflation": "(true/false, default: false) Whether to conflate drug and chemical identifiers.",
            "description": "(true/false, default: false) Whether to include descriptions in the response.",
        }

    def transform(self, annotated_text: AnnotatedText, props={}) -> AnnotatedText:
        # Set up query.
        session = self.requests_session

        ids = list(map(lambda a: a.id, annotated_text.annotations))

        response = session.post(
            self.get_normalized_nodes_url,
            json={
                "curies": ids,
                "conflate": "true"
                if props.get("geneprotein_conflation", True)
                else "false",
                "drug_chemical_conflate": "true"
                if props.get("drugchemical_conflation", False)
                else "false",
                "description": "true" if props.get("description", False) else "false",
            },
        )
        if response.status_code != 200:
            # raise Exception(f"NodeNorm returned status code {response.status_code}")
            logging.error(
                f"NodeNorm returned status code {response.status_code} {response.text} for CURIEs {ids}, skipping."
            )
            return annotated_text

        results = response.json()

        output_annotations = []
        for annotation in annotated_text.annotations:
            # No result?
            if annotation.id not in results or results[annotation.id] is None:
                output_annotations.append(annotation)
                continue

            # We have a result!
            result = results[annotation.id]
            if (
                "id" not in result
                or "identifier" not in result["id"]
                or not result["id"]["identifier"]
            ):
                # No identifier, skip.
                output_annotations.append(annotation)
                continue

            if (
                isinstance(annotation, NormalizedAnnotation)
                and result["id"]["identifier"] == annotation.curie
            ):
                # Already normalized, skip.
                output_annotations.append(annotation)
                continue

            types = result["type"]
            if not types:
                types = ["biolink:NamedThing"]

            normalized_annotation = NormalizedAnnotation.from_annotation(
                annotation,
                provenance=self.provenance,
                curie=result["id"]["identifier"],
                biolink_type=types[0],
                label=result["id"].get("label", ""),
            )
            normalized_annotation.props["types"] = types
            normalized_annotation.props["ic"] = results.get("ic", None)

            if props.get("description", False):
                normalized_annotation.props["description"] = result["id"].get(
                    "description", None
                )

            output_annotations.append(normalized_annotation)

        return AnnotatedText(annotated_text.text, output_annotations)



import pytest
from requests import HTTPError

from renci_ner.services.linkers.nameres import NameRes
from renci_ner.services.linkers.sapbert import SAPBERTAnnotator
from renci_ner.services.ner.biomegatron import BioMegatron
from renci_ner.services.normalization.nodenorm import NodeNorm


def test_multiple_annotators():
    try:
        biomegatron = BioMegatron()
    except HTTPError as err:
        pytest.skip(f"BioMegatron is not available: {err}")
        return

    nameres = NameRes()
    sapbert = SAPBERTAnnotator()
    nodenorm = NodeNorm()

    text = "The brain is part of the nervous system."
    result_nameres = (
        biomegatron.annotate(text).reannotate(nameres, {"limit": 1}).transform(nodenorm)
    )
    result_sapbert = (
        biomegatron.annotate(text).reannotate(sapbert, {"limit": 1}).transform(nodenorm)
    )

    assert result_nameres.text == text
    assert result_nameres.text == result_sapbert.text
    assert len(result_nameres.annotations) == 2
    assert len(result_nameres.annotations) == len(result_sapbert.annotations)

    # Make sure that all the annotations are identical between NodeNorm and NameRes.
    for nameres_annotation, sapbert_annotation in zip(
        result_nameres.annotations, result_sapbert.annotations
    ):
        assert nameres_annotation.text == sapbert_annotation.text
        assert nameres_annotation.id == sapbert_annotation.id
        assert nameres_annotation.label == sapbert_annotation.label
        assert nameres_annotation.type == sapbert_annotation.type

        # NameRes and BabelSAPBERT should not need to be normalized.
        assert len(nameres_annotation.provenances) == 2
        assert nameres_annotation.provenances[0] == biomegatron.provenance
        assert nameres_annotation.provenances[1] == nameres.provenance

        # Some SAPBERT annotations may need to be normalized.
        if len(sapbert_annotation.provenances) == 2:
            assert sapbert_annotation.provenances[0] == biomegatron.provenance
            assert sapbert_annotation.provenances[1] == sapbert.provenance
        else:
            assert sapbert_annotation.provenances[0] == biomegatron.provenance
            assert sapbert_annotation.provenances[1] == sapbert.provenance
            assert sapbert_annotation.provenances[2] == nodenorm.provenance



#
# An annotator service to identify biomedical concepts in plain text.
# Source code: https://github.com/RENCI-NER/nemo-serve
# Hosted at: https://med-nemo.apps.renci.org/docs
#

import requests

from renci_ner.core import AnnotatedText, Annotation, AnnotationProvenance, Annotator

# Configuration.
RENCI_BIOMEGATRON_URL = "https://med-nemo.apps.renci.org"


class BioMegatron(Annotator):
    """
    Provides an Annotator interface to a BioMegatron service.
    """

    @property
    def provenance(self) -> AnnotationProvenance:
        """Return an AnnotationProvenance describing annotations produced by this service."""
        return AnnotationProvenance(
            name="BioMegatron", url=RENCI_BIOMEGATRON_URL, version=self.openapi_version
        )

    def __init__(self, url=RENCI_BIOMEGATRON_URL, requests_session=requests.Session()):
        """
        Set up a BioMegatron service.

        :param url: The URL of the BioMegatron service.
        :param requests_session: A Requests session object to use instead of the default one.
        """
        self.url = url
        self.annotate_url = url + "/annotate/"
        self.requests_session = requests_session

        result = requests.get(self.url + "/openapi.json")
        result.raise_for_status()
        openapi_data = result.json()
        self.openapi_version = openapi_data.get("info", {"version": "NA"}).get(
            "version", "NA"
        )

    def supported_properties(self):
        return {}

    def annotate(self, text, props={}) -> AnnotatedText:
        # Set up query.
        session = self.requests_session

        response = session.post(
            self.annotate_url,
            json={
                "text": text,
                "model_name": "token_classification",
            },
        )

        response.raise_for_status()

        result = response.json()

        annotations = []
        for denotation in result.get("denotations", []):
            span = denotation.get("span", {})
            start_index = span.get("begin", -1)
            end_index = span.get("end", -1)

            annotations.append(
                Annotation(
                    text=denotation.get("text", ""),
                    start=start_index,
                    end=end_index,
                    id=denotation.get("id", ""),
                    label="",
                    type=denotation.get("obj", ""),
                    props={},
                    provenance=self.provenance,
                )
            )

        return AnnotatedText(text, annotations)



import pytest
from requests import HTTPError

from renci_ner.services.linkers.nameres import NameRes
from renci_ner.services.linkers.sapbert import SAPBERTAnnotator
from renci_ner.services.ner.biomegatron import BioMegatron
from renci_ner.services.normalization.nodenorm import NodeNorm


def test_check():
    try:
        biomegatron = BioMegatron()
    except HTTPError as err:
        pytest.skip(f"BioMegatron is not available: {err}")
        return

    nameres = NameRes()
    sapbert = SAPBERTAnnotator()
    nodenorm = NodeNorm()

    text = "Actin in the brain is part of the nervous system."
    result_nameres = (
        biomegatron.annotate(text)
        .reannotate(nameres, {"limit": 1})
        .transform(nodenorm, {"geneprotein_conflation": True})
    )
    result_sapbert = (
        biomegatron.annotate(text)
        .reannotate(sapbert, {"limit": 1})
        .transform(nodenorm, {"geneprotein_conflation": True})
    )

    assert result_nameres.text == text
    assert result_nameres.text == result_sapbert.text
    assert len(result_nameres.annotations) == 3
    assert len(result_nameres.annotations) == len(result_sapbert.annotations)

    # Let's look at the actin annotation in more detail.
    nameres_actin = result_nameres.annotations[0]
    sapbert_actin = result_sapbert.annotations[0]

    # NameRes returns
    assert nameres_actin.based_on[-1].id == "UniProtKB:P63261"
    assert nameres_actin.based_on[-1].label == "ACTG_HUMAN Actin, cytoplasmic 2 (sprot)"
    assert nameres_actin.text == "Actin"
    assert nameres_actin.id == "NCBIGene:71"
    assert nameres_actin.label == "ACTG1"
    assert nameres_actin.curie == "NCBIGene:71"
    assert nameres_actin.biolink_type == "biolink:Gene"
    assert nameres_actin.provenance == nodenorm.provenance

    # SAPBERT returns ACTIN (PANTHER.FAMILY:PTHR11937), which normalizes to itself,
    # so effectively no normalization appears to have occurred.
    assert sapbert_actin.label == "ACTIN"
    assert sapbert_actin.id == "PANTHER.FAMILY:PTHR11937"
    assert sapbert_actin.biolink_type == "biolink:GeneFamily"
    assert sapbert_actin.provenance == sapbert.provenance



import pytest

from renci_ner.core import AnnotationProvenance, NormalizedAnnotation


def test_normalized_annotations():
    provenance = AnnotationProvenance("Test", "http://example.com", "0.1.0")

    # NormalizedAnnotations without a `biolink:` prefix are not allowed!
    with pytest.raises(ValueError):
        normalized_annotation = NormalizedAnnotation(
            provenance=provenance,
            text="brain",
            id="UBERON:0000955",
            label="brain",
            type="AnatomicalEntity",
            biolink_type="AnatomicalEntity",
            start=0,
            end=4,
        )
    normalized_annotation = NormalizedAnnotation(
        provenance=provenance,
        text="brain",
        id="UBERON:0000955",
        label="brain",
        type="AnatomicalEntity",
        biolink_type="biolink:AnatomicalEntity",
        start=0,
        end=4,
    )
    with pytest.raises(ValueError):
        normalized_annotation.biolink_type = "AnatomicalEntity"



#
# A Named Entity Linker based on the Babel cliques.
# Source code: https://github.com/TranslatorSRI/NameResolution
# Hosted at: https://name-resolution-sri.renci.org/docs
#
import requests

from renci_ner.core import (
    AnnotatedText,
    AnnotationProvenance,
    Annotator,
    NormalizedAnnotation,
)

# Configuration.
RENCI_NAMERES_URL = "https://name-resolution-sri.renci.org"


class NameRes(Annotator):
    """
    A Named Entity Linker based on the Babel cliques.
    """

    @property
    def provenance(self) -> AnnotationProvenance:
        """Return an AnnotationProvenance describing annotations produced by this service."""
        return AnnotationProvenance(
            name="NameRes", url=RENCI_NAMERES_URL, version=self.openapi_version
        )

    def __init__(self, url=RENCI_NAMERES_URL, requests_session=requests.Session()):
        """
        Set up a BioMegatron service.

        :param url: The URL of the BioMegatron service.
        :param requests_session: A Requests session object to use instead of the default one.
        """
        self.url = url
        self.lookup_url = url + "/lookup"
        self.requests_session = requests_session

        response = requests.get(self.url + "/openapi.json")
        response.raise_for_status()
        openapi_data = response.json()
        self.openapi_version = openapi_data.get("info", {"version": "NA"}).get(
            "version", "NA"
        )

    def supported_properties(self):
        """Some configurable parameters."""
        return {
            "autocomplete": "(true/false, default: false) Whether to search for incomplete words (e.g. 'bra' for brain).",
            "limit": "(int, default: 10) The number of results to return.",
            # TODO: add remaining.
        }

    def annotate(self, text, props={}) -> AnnotatedText:
        # Set up query.
        session = self.requests_session

        response = session.get(
            self.lookup_url,
            params={
                "string": text,
                "autocomplete": props.get("autocomplete", "false"),
                "limit": props.get("limit", 10),
                "highlighting": props.get("highlighting", "false"),
                "biolink_type": "|".join(props.get("biolink_types", [])),
                "only_prefixes": "|".join(props.get("only_prefixes", [])),
                "exclude_prefixes": "|".join(props.get("exclude_prefixes", [])),
                "only_taxa": "|".join(props.get("only_taxa", [])),
            },
        )

        response.raise_for_status()
        results = response.json()

        annotations = []
        for result in results:
            annotations.append(
                # Since NameRes is normalized to Babel, we can treat it as a NormalizedAnnotation.
                NormalizedAnnotation(
                    text=text,
                    curie=result.get("curie", ""),
                    id=result.get("curie", ""),
                    label=result.get("label", ""),
                    biolink_type=result.get("types", ["biolink:NamedThing"])[0],
                    type=result.get("types", ["biolink:NamedThing"])[0],
                    props={
                        "score": result.get("score", 0),
                        "clique_identifier_count": result.get(
                            "clique_identifier_count", 0
                        ),
                        "synonyms": result.get("synonyms", []),
                        "highlighting": result.get("highlighting", {}),
                        "types": result.get("types", []),
                        "taxa": result.get("taxa", []),
                    },
                    provenance=self.provenance,
                    # Since we're using the whole text, let's just use that
                    # as the start/end.
                    start=0,
                    end=len(text),
                )
            )

        return AnnotatedText(text, annotations)



import re

from renci_ner.services.linkers.nameres import NameRes


def test_check():
    nameres = NameRes()
    result = nameres.annotate("brain", {"limit": 11})
    assert result.text == "brain"
    annotations = result.annotations
    assert len(annotations) == 11
    top_annot = annotations[0]
    assert top_annot.label == "brain"
    assert top_annot.id == "UBERON:0000955"
    assert top_annot.type == "biolink:GrossAnatomicalStructure"

    assert top_annot.provenance.name == "NameRes"
    assert top_annot.provenance.url == "https://name-resolution-sri.renci.org"

    # NameRes version changes quite frequently, but we can confirm that we're still in a 1.x.x version.
    assert re.compile(r"^1.\d+.\d+").match(top_annot.provenance.version)



from renci_ner.core import AnnotationProvenance
from renci_ner.services.linkers.sapbert import SAPBERTAnnotator


def test_check():
    sapbert = SAPBERTAnnotator()
    result = sapbert.annotate("brain", {"limit": 11})
    assert result.text == "brain"
    annotations = result.annotations
    assert len(annotations) == 11
    top_annot = annotations[0]
    assert top_annot.label == "brain"
    assert top_annot.id == "UBERON:0000955"
    assert top_annot.type == "biolink:GrossAnatomicalStructure"
    assert top_annot.provenance == AnnotationProvenance(
        name="BabelSAPBERT", version="0.1.0", url="https://sap-qdrant.apps.renci.org"
    )



from dataclasses import dataclass, field
from typing import Self


@dataclass
class AnnotationProvenance:
    name: str
    url: str
    version: str


@dataclass
class Annotation:
    """
    A class for storing a single annotation.
    """

    text: str
    id: str
    label: str
    type: str
    start: int
    end: int
    provenance: AnnotationProvenance
    based_on: list[Self] = field(default_factory=list)
    props: dict = field(default_factory=dict)

    @property
    def provenances(self) -> list[AnnotationProvenance]:
        return list(map(lambda ann: ann.provenance, self.based_on)) + [self.provenance]


@dataclass
class NormalizedAnnotation(Annotation):
    curie: str = None
    biolink_type: str = None

    def __setattr__(self, name, value):
        """Validate biolink_type."""
        if (
            name == "biolink_type"
            and value is not None
            and not value.lower().startswith("biolink:")
        ):
            raise ValueError(
                f"Invalid biolink_type: must start with 'biolink:' but got '{value}'."
            )

        # TODO: it'd probably be a good idea to check formatting for CURIEs as well, but that's less well defined.

        # No problems.
        super().__setattr__(name, value)

    @classmethod
    def from_annotation(
        cls,
        annotation: Annotation,
        provenance: AnnotationProvenance,
        curie=None,
        biolink_type=None,
        label=None,
    ) -> Self:
        """
        Creates an instance of NormalizedAnnotation from the provided Annotation object.

        This method serves as a factory method to instantiate a NormalizedAnnotation
        object using the given Annotation instance. Additional optional properties,
        such as curie and biolink_type, may also be provided to customize the
        generated object. The method utilizes the attributes from the Annotation
        instance to populate the corresponding fields in the NormalizedAnnotation.

        :param annotation: The Annotation instance used as the basis for constructing
            the NormalizedAnnotation object.
        :type annotation: Annotation
        :param provenance: The provenance of the normalized annotation.
        :type provenance: AnnotationProvenance
        :param curie: Optional CURIE string for additional annotation details. If not
            provided, it defaults to None.
        :type curie: str, optional
        :param biolink_type: Optional biolink type string to specify the type of the
            annotation. If not provided, it defaults to None.
        :type biolink_type: str, optional
        :param label: Optional label string to specify the label of the annotation. Will overwrite the
            annotation label if one is provided.
        :type label: str, optional
        :return: A new instance of NormalizedAnnotation initialized with the provided
            annotation and optional parameters.
        :rtype: Self
        """

        if label is None:
            label = annotation.label

        if curie is None:
            curie = annotation.id

        if biolink_type is None:
            biolink_type = annotation.type

        return NormalizedAnnotation(
            text=annotation.text,
            start=annotation.start,
            end=annotation.end,
            provenance=provenance,
            based_on=[*annotation.based_on, annotation],
            props=annotation.props,
            # These fields are overwritten during normalization.
            id=curie,
            curie=curie,
            label=label,
            type=biolink_type,
            biolink_type=biolink_type,
        )


class AnnotatedText:
    """
    A class for storing a text along with a set of annotations from a single source.
    """

    def __init__(self, text: str, annotations: list[Annotation]) -> None:
        self.text = text
        self.annotations = annotations

    def transform(self, transformer: "Transformer", props: dict = {}) -> Self:
        """
        Transform the annotations in this AnnotatedText with a transformer.

        Note that in this case the Transformer is responsible for updating the provenance of the transformed annotations.

        :param transformer: A Transformer to transform this annotated text.
        :param props: The properties to pass to the transformer.
        :return: The transformed AnnotatedText.
        """
        return transformer.transform(self, props)

    def reannotate(self, annotator: "Annotator", props: dict = {}) -> Self:
        """
        Reannotate the annotations in this AnnotatedText with another annotator.

        Note that this does NOT mean that the original text is reannotated -- rather, each individual annotation
        will be annotated by the next annotator. This allows us to standardize some common situations:
        - If an annotation could not be annotated by the next annotator, it will be left as-is.
        - If an annotation is annotated by the next annotator with a single annotation, we will replace the previous
          annotation with this annotation, but update provenance and based_on fields.
        - If an annotation is annotated by the next annotator with multiple annotations, we will replace this
          annotation with all of those annotations, with provenance and based_on fields updated appropriately.

        :param annotator: The other annotator to annotate these annotations with.
        :type annotator: Annotator
        :param props: A dictionary of properties to pass to the annotator.
        :return: AnnotatedText with annotations re-annotated with the other annotator.
        """

        new_annotations = []
        for annotation in self.annotations:
            annotated_text = annotator.annotate(annotation.text, props=props)
            reannotations = annotated_text.annotations
            if len(reannotations) == 0:
                # Leave the current annotation unchanged.
                new_annotations.append(annotation)
            else:
                # We have one or more annotations. So we need to update the based_on by adding annotation to the
                # existing list.
                new_based_on = [*annotation.based_on, annotation]
                base_start = annotation.start
                base_end = annotation.end

                for reannotation in reannotations:
                    # Fix the start and end indices.
                    reannotation.start = reannotation.start + base_start
                    reannotation.end = reannotation.end + base_end

                    reannotation.provenance = annotator.provenance
                    reannotation.based_on = new_based_on

                    new_annotations.append(reannotation)

        return AnnotatedText(self.text, new_annotations)


class Annotator:
    """
    An interface for annotating text using a service.
    """

    @property
    def provenance(self) -> AnnotationProvenance:
        """
        Return an AnnotationProvenance describing annotations produced by this service.

        :return AnnotationProvenance: The provenance of annotations generated by this Annotator.:
        """
        return AnnotationProvenance(
            name="Annotator",
            url="http://example.org/",
            version="0.0.1",
        )

    def annotate(self, text: str, props: dict = {}) -> AnnotatedText:
        """
        Annotate a text with a set of service-specific properties.

        :return AnnotatedText: The annotated text.
        """
        pass

    def supported_properties(self) -> dict[str, str]:
        """
        Return a list of supported properties for this service.

        :return: A dictionary of supported properties, with the values describing each property.
        """
        return {}


class Transformer:
    """
    An interface for transforming annotated text.
    """

    def supported_properties(self) -> dict[str, str]:
        """
        Return a list of supported properties for this service.

        :return: A dictionary of supported properties, with the values describing each property.
        """
        return {}

    def transform(self, annotated_text: AnnotatedText, props={}) -> AnnotatedText:
        """
        Transform an annotated text into a new annotated text.

        Note that in this case the Transformer is responsible for updating the provenance of the transformed annotations.

        :param annotated_text: The annotated text to transform.
        :param props: Properties supported by this transformer to use during the transformation.
        :return: The transformed AnnotatedText.
        """
        return annotated_text



import pytest
from requests import HTTPError

from renci_ner.core import AnnotationProvenance
from renci_ner.services.ner.biomegatron import BioMegatron


def test_check():
    try:
        biomegatron = BioMegatron()
    except HTTPError as err:
        pytest.skip(f"BioMegatron is not available: {err}")
        return

    query = "The brain is a significant part of the nervous system."
    result = biomegatron.annotate(query)
    assert result.text == query
    annotations = result.annotations
    assert len(annotations) == 2

    brain = annotations[0]
    assert brain.label == ""
    assert brain.type == "biolink:AnatomicalEntity"
    assert len(brain.provenances) == 1
    assert brain.provenances[0] == AnnotationProvenance(
        name="BioMegatron", version="0.1.0", url="https://med-nemo.apps.renci.org"
    )

    nervous_system = annotations[1]
    assert nervous_system.label == ""
    assert nervous_system.type == "biolink:AnatomicalEntity"
    assert len(nervous_system.provenances) == 1
    assert nervous_system.provenances[0] == AnnotationProvenance(
        name="BioMegatron", version="0.1.0", url="https://med-nemo.apps.renci.org"
    )
